'''

 Filter to implement a SKETCH look of the INPUT IMAGE
 (following a tutorial of Jamack)
 If chosen by the user, 3 different images are produced
 ...................................................... 
 
 Release 1.0   initial
 
Flow implemented to get the final result:      
 
  1. create a new image with dimension chosen by the user
  2. create a BG as a copy of the source drawable
  3. duplicate and apply Pseudo Freaky B&W; set mode=Dodge
  4. duplicate, invert and apply Gimp Erode (twice) and Unsharp Mask (twice)
  5. create a new layer from visible; set mode=Multiply
  6. create a new layer and fill it with a Paper texture with layer mode = COLOR
  7. duplicate the BG and set it on top; set mode=GRAIN_MERGE
  8. create a new layer from visible; set mode=Normal; name it Sketch1
     if the user choice for the aspect is "Smooth":
     - create a copy of the BG, place it on top and set mode to SCREEN
     
 now the filter procedes with the additional modes
  9. set visibility off on Sketch1
 10. change mode on duplicated BG; set it to HARDLIGHT2
 11. create a new layer from visible; set mode=Normal; name it Sketch2
     if the user choice for the aspect is "Smooth":
     - create a copy of the BG, place it on top and set mode to SCREEN
 12. set visibility off on Sketch2
 13. change mode on duplicated BG; set it to OVERLAY
 14. create a new layer from visible; set mode=Normal; name it Sketch3
     if the user choice for the aspect is "Smooth":
     - create a copy of the BG, place it on top and set mode to SCREEN
 15. set visibility off on Sketch3
 16. create a new layer from visible; set mode=Normal; name it Sketch4
 17. change mode on duplicated BG; set it to BURN
     if the user choice for the aspect is "Smooth":
     - create a copy of the BG, place it on top and set mode to SCREEN
  
 18. if chosen by the user, create and display 4 images, using the 4 layers:
     Sketch1, Sketch2, Sketch3, Sketch4 
 ----------------------------------------------------------------
 COPYRIGHT NOTICE
 ----------------------------------------------------------------
 
 This program is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 3 of the License, or
 (at your option) any later version.  
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, you can view the GNU General Public
 License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
 Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
 Ave, Cambridge, MA 02139, USA.
 
 -------------------------------------------------------------------
 other info
 -------------------------------------------------------------------
 the following line terminates the long initial comments
 
'''
#!/usr/bin/env python
from gimpfu import *
import decimal

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
############################################################################
#
def fakeFreakyBW (img, drw):
    pdb.gimp_desaturate(drw)
    clouds = drw.copy()
    clouds.mode = SCREEN_MODE
    clouds.name = "Clouds Layer"
    clouds.opacity = 50.0 
    img.add_layer(clouds, 0)
    pdb.gimp_edit_fill (clouds, WHITE_FILL)
    pdb.plug_in_plasma (img, clouds, 3, 0.3)
    pdb.gimp_desaturate(clouds)
    pdb.plug_in_gauss_iir2 (img, clouds, img.width/50, img.height/50)
    clouds = pdb.gimp_image_get_active_layer (img)
    odrw = pdb.gimp_image_merge_down (img, clouds, 0)
    pdb.plug_in_autostretch_hsv (img, odrw)
    pdb.plug_in_unsharp_mask (img, odrw, 50.0, 0.2, 0)     # increase contrast on details
    return(odrw)

def JamackSketchSE(inImage, inDrawable,
                   widthheight,
                   aspect,
                   doCreate) :

    pdb.gimp_context_push
    currFG = pdb.gimp_context_get_foreground()
    currBG = pdb.gimp_context_get_background()
    
    width = inImage.width
    height = inImage.height
    type = inImage.base_type
    
    name = inDrawable.name
    if '.' in name:
        split = name.split(".", 1)
        name = split[0]
    suffix = ", as a Sketch"
    
    # calculate scale factor for the new output image
    if widthheight == 0 :   # means the user wants to have same size as input
        scale = 1
    else :     
        optiWidthHeight = widthheight       # size to work on to get best results
        realWidthHeight = width+height      # real size of the input image
        scale = decimal.Decimal(optiWidthHeight)/decimal.Decimal(realWidthHeight)      # ratio for scaling

    newwidth=int(width*scale)
    newheight=int(height*scale)
    
    #info_msg ("newwidth="+str(newwidth))
    newImage = pdb.gimp_image_new (newwidth, newheight, type)                    
    newBG = pdb.gimp_layer_new_from_drawable(inDrawable, newImage)     # background layer
    newBG.mode = NORMAL_MODE
    newBG.name = "Background Image Layer"
    newBG.opacity = 100.0 
    newImage.add_layer (newBG, 0)
    pdb.gimp_layer_scale(newBG, newwidth, newheight, False)
    alpha=pdb.gimp_drawable_has_alpha(newBG)
    if (alpha==False):
        pdb.gimp_layer_add_alpha (newBG)    #  RGBA
    
    # create the working layers
    
    FreakyBW = newBG.copy()
    FreakyBW.mode = NORMAL_MODE
    FreakyBW.name = "FreakyBW Layer"
    FreakyBW.opacity = 100.0 
    newImage.add_layer(FreakyBW, 0)
    FreakyBW = fakeFreakyBW (newImage, FreakyBW)
    
    #pdb.gimp_display_new (newImage)
    #return()
    
    Inverted = FreakyBW.copy()
    Inverted.mode = DODGE_MODE
    Inverted.name = "Inverted Layer"
    Inverted.opacity = 100.0 
    newImage.add_layer(Inverted, 0)
    pdb.gimp_invert (Inverted)
    pdb.plug_in_erode (newImage, Inverted, 1, HISTOGRAM_VALUE, 1.0, 7, 0, 128)
    pdb.plug_in_erode (newImage, Inverted, 1, HISTOGRAM_VALUE, 1.0, 7, 0, 128)
    pdb.plug_in_unsharp_mask (newImage, FreakyBW, 1.0, 1.0, 0)     # increase contrast on positive details
    pdb.plug_in_unsharp_mask (newImage, FreakyBW, 1.0, 1.0, 0)     # increase contrast on positive details
    
    #pdb.gimp_display_new (newImage)
    #return()
    
    Visible = pdb.gimp_layer_new_from_visible(newImage, newImage, "Visible Layer")
    newImage.add_layer(Visible, 0)
    pdb.gimp_layer_set_opacity(Visible, 100.0)
    pdb.gimp_layer_set_mode(Visible, MULTIPLY_MODE)
    pdb.gimp_levels(Visible, HISTOGRAM_VALUE, 0, 255, 0.15, 0, 255)
    
    Paper = pdb.gimp_layer_new(newImage, newwidth, newheight, RGBA_IMAGE, "Paper Layer", 100.0, COLOR_MODE)
    newImage.add_layer(Paper, 0)
    pdb.gimp_context_set_pattern("Paper_Creme anglaise taster.jpg")
    pdb.gimp_edit_fill(Paper, PATTERN_FILL)
    
    #pdb.gimp_display_new (newImage)
    #return()
    
    newBG2 = newBG.copy()
    newBG2.name = "dup BG Layer"
    newBG2.opacity = 80.0 
    newImage.add_layer(newBG2, 0)
    
    #pdb.gimp_display_new (newImage)
    #return()
        
    # First Outcome:
    newBG2.mode = GRAIN_MERGE_MODE
    newBG2.opacity = 85.0 
    Sketch1 = pdb.gimp_layer_new_from_visible(newImage, newImage, name+suffix+" (Grain-Merge) (raw)")
    newImage.add_layer(Sketch1, 0)
    pdb.gimp_layer_set_opacity(Sketch1, 100.0)
    if aspect == 1 : # soft
        Sketch1copy = newBG.copy()
        newImage.add_layer(Sketch1copy, 0)
        pdb.gimp_layer_set_mode(Sketch1copy, SCREEN_MODE)
        pdb.plug_in_gauss_iir2 (newImage, Sketch1copy, newwidth/10, newheight/10)
        Sketch1 = pdb.gimp_image_merge_down (newImage, Sketch1copy, 0)
        Sketch1.name = name+suffix+" (Grain-Merge) (smoothed)"
    
    # Second Outcome:  
    pdb.gimp_item_set_visible (Sketch1, False)
    newBG2.mode = HARDLIGHT_MODE
    newBG2.opacity = 75.0 
    Sketch2 = pdb.gimp_layer_new_from_visible(newImage, newImage, name+suffix+" (Hard-Light) (raw)")
    newImage.add_layer(Sketch2, 1)
    pdb.gimp_layer_set_opacity(Sketch2, 100.0)
    if aspect == 1 : # soft
        Sketch2copy = newBG.copy()
        newImage.add_layer(Sketch2copy, 1)
        pdb.gimp_layer_set_mode(Sketch2copy, SCREEN_MODE)
        pdb.plug_in_gauss_iir2 (newImage, Sketch2copy, newwidth/10, newheight/10)
        Sketch2 = pdb.gimp_image_merge_down (newImage, Sketch2copy, 0)
        Sketch2.name = name+suffix+" (Hard-Light) (smoothed)"
    
    # Third Outcome:  
    pdb.gimp_item_set_visible (Sketch2, False)
    newBG2.mode = OVERLAY_MODE
    newBG2.opacity = 100.0 
    Sketch3 = pdb.gimp_layer_new_from_visible(newImage, newImage, name+suffix+" (Overlay) (raw)")
    newImage.add_layer(Sketch3, 2)
    pdb.gimp_layer_set_opacity(Sketch3, 100.0)
    if aspect == 1 : # soft
        Sketch3copy = newBG.copy()
        newImage.add_layer(Sketch3copy, 2)
        pdb.gimp_layer_set_mode(Sketch3copy, SCREEN_MODE)
        pdb.plug_in_gauss_iir2 (newImage, Sketch3copy, newwidth/10, newheight/10)
        Sketch3 = pdb.gimp_image_merge_down (newImage, Sketch3copy, 0)
        Sketch3.name = name+suffix+" (Overlay) (smoothed)"
    
    # Fourth Outcome:  
    pdb.gimp_item_set_visible (Sketch3, False)
    newBG2.mode = BURN_MODE
    newBG2.opacity = 50.0 
    Sketch4 = pdb.gimp_layer_new_from_visible(newImage, newImage, name+suffix+" (Burn) (raw)")
    newImage.add_layer(Sketch4, 3)
    pdb.gimp_layer_set_opacity(Sketch4, 100.0)
    if aspect == 1 : # soft
        Sketch4copy = newBG.copy()
        newImage.add_layer(Sketch4copy, 3)
        pdb.gimp_layer_set_mode(Sketch4copy, SCREEN_MODE)
        pdb.plug_in_gauss_iir2 (newImage, Sketch4copy, newwidth/10, newheight/10)
        Sketch4 = pdb.gimp_image_merge_down (newImage, Sketch4copy, 0)
        Sketch4.name = name+suffix+" (Burn) (smoothed)"
    
    pdb.gimp_item_set_visible (Sketch1, True)
    pdb.gimp_item_set_visible (Sketch2, True)
    pdb.gimp_item_set_visible (Sketch3, True)
    
    pdb.gimp_image_set_active_layer (newImage, Sketch1)
    
    if doCreate == True :
        # first new image 
        Out1Image = pdb.gimp_image_new (newwidth, newheight, type)                    
        Out1BG = pdb.gimp_layer_new_from_drawable(Sketch1, Out1Image)     # background layer
        Out1BG.mode = NORMAL_MODE
        Out1BG.name = Sketch1.name
        Out1BG.opacity = 100.0 
        Out1Image.add_layer (Out1BG, 0)
        pdb.gimp_display_new (Out1Image)
        # second new image 
        Out2Image = pdb.gimp_image_new (newwidth, newheight, type)                    
        Out2BG = pdb.gimp_layer_new_from_drawable(Sketch2, Out2Image)     # background layer
        Out2BG.mode = NORMAL_MODE
        Out2BG.name = Sketch2.name
        Out2BG.opacity = 100.0 
        Out2Image.add_layer (Out2BG, 0)
        pdb.gimp_display_new (Out2Image)
        # third new image 
        Out3Image = pdb.gimp_image_new (newwidth, newheight, type)                    
        Out3BG = pdb.gimp_layer_new_from_drawable(Sketch3, Out3Image)     # background layer
        Out3BG.mode = NORMAL_MODE
        Out3BG.name = Sketch3.name
        Out3BG.opacity = 100.0 
        Out3Image.add_layer (Out3BG, 0)
        pdb.gimp_display_new (Out3Image)
        # fourth new image 
        Out4Image = pdb.gimp_image_new (newwidth, newheight, type)                    
        Out4BG = pdb.gimp_layer_new_from_drawable(Sketch4, Out4Image)     # background layer
        Out4BG.mode = NORMAL_MODE
        Out4BG.name = Sketch4.name
        Out4BG.opacity = 100.0 
        Out4Image.add_layer (Out4BG, 0)
        pdb.gimp_display_new (Out4Image)
    
    pdb.gimp_display_new (newImage)
    pdb.gimp_displays_flush
   
    # Close the undo group.
    pdb.gimp_context_pop
    pdb.gimp_context_set_foreground(currFG)
    pdb.gimp_context_set_background(currBG)
    
    return(Sketch1)

# This is the plugin registration function
register(
    "JamackSketchSE",    
    "to set a sketch look in 4 different modes on an image",   
    "This script sets a sketch look in 4 different modes on an image (inspired by a Jamack tut).",
    "Diego", 
    "Diego Nassetti ", 
    "November 2016",
    "JamackSketchSE", 
    "RGB*", 
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "idrawable", "Input drawable", None),
      (PF_ADJUSTMENT, "size", "Width+Height of New Image (0=same as input)", 0, (0, 9600, 300)),
      (PF_OPTION, "aspect", "Outcome aspect", 1, ("Raw", "Smooth")),
      (PF_TOGGLE, "create4", "Create 4 Images?", True),
    ], 
    [
      (PF_DRAWABLE, "odrawable", "Output drawable", None),  
    ],
    JamackSketchSE,
    menu="<Image>/Diego/Artistic",
    )

main()
